//+------------------------------------------------------------------+
//|                                                   TrailingCT.mqh |
//|                   Copyright 2009-2013, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <Math\Stat\Math.mqh>
#include <Expert\ExpertTrailing.mqh>
#include <ct_18.mqh>
#define __PHI 0.61803398874989
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Trailing Stop based on 'Category Theory' Natural Transformations (part 1)|
//| Type=Trailing                                                    |
//| Name=CategoryTheory                                              |
//| ShortName=CT                                                     |
//| Class=CTrailingCT                                                |
//| Page=trailing_ct                                                 |
//| Parameter=Step,double,0.5,Trailing Step                          |
//| Parameter=Decay,double,0.001,Multi Layer Perceptron Decay        |
//| Parameter=Restarts,int,2,Multi Layer Perceptron Restarts         |
//| Parameter=AB_Hidden,int,2,AB Hidden Layer Size                   |
//| Parameter=CD_Hidden,int,2,CD Hidden Layer Size                   |
//| Parameter=AC_Hidden,int,2,AC Hidden Layer Size                   |
//| Parameter=BD_Hidden,int,2,BD Hidden Layer Size                   |
//| Parameter=ExtraTraining,int,0,Extra Training                     |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CTrailingCT.                                               |
//| Appointment: Class traling stops with 'Category Theory'          |
//|               Natural Transformations (part 1).                  |
//| Derives from class CExpertTrailing.                              |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CTrailingCT : public CExpertTrailing
  {
protected:
   //--- adjusted parameters
   double m_step;                // trailing step
   double m_decay;               // decay
   int m_restarts;               // restarts
   
   int m_ab_hidden;              //
   int m_cd_hidden;              //
   int m_ac_hidden;              //
   int m_bd_hidden;              //
   
   int m_extra_training;         //
   
   CiATR m_atr;
   CiBands m_bands;
   //

public:
   //--- methods of setting adjustable parameters
   
   //--- method of verification of settings
   virtual bool         ValidationSettings(void);
   //--- method of creating the indicator and timeseries
   virtual bool         InitIndicators(CIndicators *indicators);
   //--- methods of checking if the market models are formed
   virtual bool         CheckTrailingStopLong(CPositionInfo *position,double &sl,double &tp);
   virtual bool         CheckTrailingStopShort(CPositionInfo *position,double &sl,double &tp);
   //---
                        CTrailingCT(void);
                        ~CTrailingCT(void);
   //--- methods of initialization of protected data
   void                 Step(double value)            { m_step=value;         }
   void                 Decay(double value)           { m_decay=value;        }
   void                 Restarts(int value)           { m_restarts=value;     }
   
   void                 AB_Hidden(int value)          { m_ab_hidden=value;    }
   void                 CD_Hidden(int value)          { m_cd_hidden=value;    }
   void                 AC_Hidden(int value)          { m_ac_hidden=value;    }
   void                 BD_Hidden(int value)          { m_bd_hidden=value;    }
   
   void                 ExtraTraining(int value)      { m_extra_training=value;    }

protected:
   //--- method of initialization of the oscillator
   bool                 InitBands(CIndicators *indicators);
   bool                 InitATR(CIndicators *indicators);

      CNaturalitySquare m_naturality_square;
      CMLPBase          m_base;
      CMLPTrain         m_train;
      CDomain<string>   m_d_atr,m_d_bands;
      CCategory         m_domain,m_codomain;
      
      double            Refresh();
      
  };
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CTrailingCT::CTrailingCT(void) : m_step(0.5),
                                 m_decay(0.001),
                                 m_restarts(2),
                                 m_ab_hidden(2),
                                 m_cd_hidden(2),
                                 m_ac_hidden(2),
                                 m_bd_hidden(2),
                                 m_extra_training(0)
  {
//--- initialization of protected data
      m_used_series=USE_SERIES_TIME+USE_SERIES_SPREAD+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE;
      
      CElement<string> _e;
      m_d_atr.Cardinality(1);
      m_d_bands.Cardinality(1);
      //
      m_naturality_square.A.Cardinality(m_extra_training+1);
      m_naturality_square.B.Cardinality(m_extra_training+1);
      m_naturality_square.C.Cardinality(m_extra_training+1);
      m_naturality_square.D.Cardinality(m_extra_training+1);
      //
      for(int i=0;i<21;i++)
      {
         _e.Let();_e.Cardinality(1);
         _e.Set(0,IntegerToString((i*10)-100));
         
         m_d_atr.Set(i,_e);
         m_d_bands.Set(i,_e);
      }
      
      m_domain.Domains(2);
      m_domain.SetDomain(0,m_d_atr);
      m_domain.SetDomain(1,m_d_bands);
      
      m_codomain.Domains(4);
      m_codomain.SetDomain(0,m_naturality_square.A);
      m_codomain.SetDomain(1,m_naturality_square.B);
      m_codomain.SetDomain(2,m_naturality_square.C);
      m_codomain.SetDomain(3,m_naturality_square.D);
      
      m_naturality_square.AB.hidden_size=m_ab_hidden;
      m_naturality_square.AB.Transform(m_naturality_square.A,m_naturality_square.B);
      
      m_naturality_square.CD.hidden_size=m_cd_hidden;
      m_naturality_square.CD.Transform(m_naturality_square.C,m_naturality_square.D);
      
      m_naturality_square.hidden_size_ac=m_ac_hidden;
      m_base.MLPCreate1(m_naturality_square.A.Cardinality(),m_naturality_square.hidden_size_ac+fmax(m_naturality_square.A.Cardinality(),m_naturality_square.C.Cardinality()),m_naturality_square.C.Cardinality(),m_naturality_square.AC);
      
      m_naturality_square.hidden_size_bd=m_bd_hidden;
      m_base.MLPCreate1(m_naturality_square.B.Cardinality(),m_naturality_square.hidden_size_bd+fmax(m_naturality_square.B.Cardinality(),m_naturality_square.D.Cardinality()),m_naturality_square.D.Cardinality(),m_naturality_square.BD);
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CTrailingCT::~CTrailingCT(void)
  {
  }
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//+------------------------------------------------------------------+
bool CTrailingCT::ValidationSettings(void)
  {
//--- validation settings of additional filters
   if(!CExpertTrailing::ValidationSettings())
      return(false);

      
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//+------------------------------------------------------------------+
bool CTrailingCT::InitIndicators(CIndicators *indicators)
  {
//--- check pointer
   if(indicators==NULL)
      return(false);
//--- initialization of indicators and timeseries of additional filters
   if(!CExpertTrailing::InitIndicators(indicators))
      return(false);
      
//--- create and initialize Bollinger Bands
   if(!InitBands(indicators))
      return(false);
      
//--- create and initialize ATR
   if(!InitATR(indicators))
      return(false);
      
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Initialize Bollinger Bands.                                      |
//+------------------------------------------------------------------+
bool CTrailingCT::InitBands(CIndicators *indicators)
  {
//--- check pointer
   if(indicators==NULL)
      return(false);
//--- add object to collection
   if(!indicators.Add(GetPointer(m_bands)))
     {
      printf(__FUNCTION__+": error adding bands pointer");
      return(false);
     }
//--- initialize object
   if(!m_bands.Create(m_symbol.Name(),m_period,20,0,2.0,PRICE_CLOSE))
     {
      printf(__FUNCTION__+": error initializing bands indicator");
      return(false);
     }
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Initialize ATR.                                                  |
//+------------------------------------------------------------------+
bool CTrailingCT::InitATR(CIndicators *indicators)
  {
//--- check pointer
   if(indicators==NULL)
      return(false);
//--- add object to collection
   if(!indicators.Add(GetPointer(m_atr)))
     {
      printf(__FUNCTION__+": error adding atr pointer");
      return(false);
     }
//--- initialize object
   if(!m_atr.Create(m_symbol.Name(),m_period,20))
     {
      printf(__FUNCTION__+": error initializing atr indicator");
      return(false);
     }
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Checking trailing stop and/or profit for long position.          |
//+------------------------------------------------------------------+
bool CTrailingCT::CheckTrailingStopLong(CPositionInfo *position,double &sl,double &tp)
  {
//--- check
      if(position==NULL)
         return(false);
      
      m_high.Refresh(-1);
      m_low.Refresh(-1);
      
      m_time.Refresh(-1);
      
      int _x=StartIndex();
      
      double _type=0.5;
      
      double _atr=fmax(2.0*m_spread.GetData(_x)*m_symbol.Point(),m_high.GetData(_x)-m_low.GetData(_x))*(_type);
      
      double _sl=m_low.GetData(_x)-(m_step*_atr);
      
      double level =NormalizeDouble(m_symbol.Bid()-m_symbol.StopsLevel()*m_symbol.Point(),m_symbol.Digits());
      double new_sl=NormalizeDouble(_sl,m_symbol.Digits());
      double pos_sl=position.StopLoss();
      double base  =(pos_sl==0.0) ? position.PriceOpen() : pos_sl;
      
      sl=EMPTY_VALUE;
      tp=EMPTY_VALUE;
      if(new_sl>base && new_sl<level)
         sl=new_sl;
      
//---
   return(sl!=EMPTY_VALUE);
  }
//+------------------------------------------------------------------+
//| Checking trailing stop and/or profit for short position.         |
//+------------------------------------------------------------------+
bool CTrailingCT::CheckTrailingStopShort(CPositionInfo *position,double &sl,double &tp)
  {
//--- check
      if(position==NULL)
         return(false);
      
      m_high.Refresh(-1);
      m_low.Refresh(-1);
      
      m_time.Refresh(-1);
      
      int _x=StartIndex();
      
      double _type=0.5;
   
      double _atr=fmax(2.0*m_spread.GetData(_x)*m_symbol.Point(),m_high.GetData(_x)-m_low.GetData(_x))*(_type);
      
      double _sl=m_high.GetData(_x)+(m_step*_atr);
      
      double level =NormalizeDouble(m_symbol.Ask()+m_symbol.StopsLevel()*m_symbol.Point(),m_symbol.Digits());
      double new_sl=NormalizeDouble(_sl,m_symbol.Digits());
      double pos_sl=position.StopLoss();
      double base  =(pos_sl==0.0) ? position.PriceOpen() : pos_sl;
      
      sl=EMPTY_VALUE;
      tp=EMPTY_VALUE;
      if(new_sl<base && new_sl>level)
         sl=new_sl;
      
//---
      return(sl!=EMPTY_VALUE);
  }
//+------------------------------------------------------------------+
//| Refresh function for naturality square.                          |
//+------------------------------------------------------------------+
double CTrailingCT::Refresh()
   {
      double _refresh=0.0;
      
      m_high.Refresh(-1);
      m_low.Refresh(-1);
      
      int _x=StartIndex();
      
      // atr domains capture 1 bar ranges       
      // bands' domains capture 2 bar ranges    
      // 1 functors capture ranges after 1 bar  
      // 2 functors capture ranges after 2 bars 
      
      int _info_ab=0,_info_bd=0,_info_ac=0,_info_cd=0;
      CMLPReport _report_ab,_report_bd,_report_ac,_report_cd;
      CMatrixDouble _xy_ab;_xy_ab.Resize(m_extra_training+1,1+1);
      CMatrixDouble _xy_bd;_xy_bd.Resize(m_extra_training+1,1+1);
      CMatrixDouble _xy_ac;_xy_ac.Resize(m_extra_training+1,1+1);
      CMatrixDouble _xy_cd;_xy_cd.Resize(m_extra_training+1,1+1);
      
      CElement<string> _e;
      for(int i=0;i<m_extra_training+1;i++)
      {
         double _a=((m_high.GetData(i+_x)-m_low.GetData(i+_x))-(m_high.GetData(i+_x+1)-m_low.GetData(i+_x+1)))/((m_high.GetData(i+_x)-m_low.GetData(i+_x))+(m_high.GetData(i+_x+1)-m_low.GetData(i+_x+1)));
         double _c=((m_high.GetData(i+_x)-m_low.GetData(i+_x))-(m_high.GetData(i+_x+2)-m_low.GetData(i+_x+2)))/((m_high.GetData(i+_x)-m_low.GetData(i+_x))+(m_high.GetData(i+_x+2)-m_low.GetData(i+_x+2)));
         double _b=((fmax(m_high.GetData(i+_x),m_high.GetData(i+_x+1))-fmin(m_low.GetData(i+_x),m_low.GetData(i+_x+1)))
                     -(fmax(m_high.GetData(i+_x+2),m_high.GetData(i+_x+3))-fmin(m_low.GetData(i+_x+2),m_low.GetData(i+_x+3))))
                     /((fmax(m_high.GetData(i+_x),m_high.GetData(i+_x+1))-fmin(m_low.GetData(i+_x),m_low.GetData(i+_x+1)))
                     +(fmax(m_high.GetData(i+_x+2),m_high.GetData(i+_x+3))-fmin(m_low.GetData(i+_x+2),m_low.GetData(i+_x+3))));
         double _d=((fmax(m_high.GetData(i+_x),m_high.GetData(i+_x+1))-fmin(m_low.GetData(i+_x),m_low.GetData(i+_x+1)))
                     -(fmax(m_high.GetData(i+_x+3),m_high.GetData(i+_x+4))-fmin(m_low.GetData(i+_x+3),m_low.GetData(i+_x+4))))
                     /((fmax(m_high.GetData(i+_x),m_high.GetData(i+_x+1))-fmin(m_low.GetData(i+_x),m_low.GetData(i+_x+1)))
                     +(fmax(m_high.GetData(i+_x+3),m_high.GetData(i+_x+4))-fmin(m_low.GetData(i+_x+3),m_low.GetData(i+_x+4))));
         
         _e.Let();_e.Cardinality(1);
         _e.Set(0,DoubleToString(_a));
         m_naturality_square.A.Set(i,_e);
         
         _e.Let();_e.Cardinality(1);
         _e.Set(0,DoubleToString(_c));
         m_naturality_square.C.Set(i,_e);
         
         _e.Let();_e.Cardinality(1);
         _e.Set(0,DoubleToString(_b));
         m_naturality_square.B.Set(i,_e);
         
         _e.Let();_e.Cardinality(1);
         _e.Set(0,DoubleToString(_d));
         m_naturality_square.D.Set(i,_e);
         
         if(i<m_extra_training+1)
         {
            _xy_ab[i].Set(0,_a);//in
            _xy_ab[i].Set(1,_b);//out
            
            _xy_bd[i].Set(0,_b);//in
            _xy_bd[i].Set(1,_d);//out
            
            _xy_ac[i].Set(0,_a);//in
            _xy_ac[i].Set(1,_c);//out
            
            _xy_cd[i].Set(0,_c);//in
            _xy_cd[i].Set(1,_d);//out
         }
      }
      
      m_train.MLPTrainLM(m_naturality_square.AB.transformer,_xy_ab,m_extra_training+1,m_decay,m_restarts,_info_ab,_report_ab);
      m_train.MLPTrainLM(m_naturality_square.BD,_xy_bd,m_extra_training+1,m_decay,m_restarts,_info_bd,_report_bd);
      m_train.MLPTrainLM(m_naturality_square.AC,_xy_ac,m_extra_training+1,m_decay,m_restarts,_info_ac,_report_ac);
      m_train.MLPTrainLM(m_naturality_square.CD.transformer,_xy_cd,m_extra_training+1,m_decay,m_restarts,_info_cd,_report_cd);
      //
      if(_info_ab>0 && _info_bd>0 && _info_ac>0 && _info_cd>0)
      {
         double _a=((m_high.GetData(_x)-m_low.GetData(_x))-(m_high.GetData(_x+1)-m_low.GetData(_x+1)))/((m_high.GetData(_x)-m_low.GetData(_x))+(m_high.GetData(_x+1)-m_low.GetData(_x+1)));
         
         double _x_ab[],_y_ab[]; ArrayResize(_x_ab,1); ArrayResize(_y_ab,1);
         ArrayInitialize(_x_ab,0.0); ArrayInitialize(_y_ab,0.0);
         //
         _x_ab[0]=_a; m_base.MLPProcess(m_naturality_square.AB.transformer,_x_ab,_y_ab);
         
         double _x_bd[],_y_bd[]; ArrayResize(_x_bd,1); ArrayResize(_y_bd,1);
         ArrayInitialize(_x_bd,0.0); ArrayInitialize(_y_bd,0.0);
         //
         _x_bd[0]=_y_ab[0]; m_base.MLPProcess(m_naturality_square.BD,_x_bd,_y_bd);
         
         double _x_ac[],_y_ac[]; ArrayResize(_x_ac,1); ArrayResize(_y_ac,1);
         ArrayInitialize(_x_ac,0.0); ArrayInitialize(_y_ac,0.0);
         //
         _x_ac[0]=_a; m_base.MLPProcess(m_naturality_square.AC,_x_ac,_y_ac);
         
         double _x_cd[],_y_cd[]; ArrayResize(_x_cd,1); ArrayResize(_y_cd,1);
         ArrayInitialize(_x_cd,0.0); ArrayInitialize(_y_cd,0.0);
         //
         _x_cd[0]=_y_ac[0]; m_base.MLPProcess(m_naturality_square.CD.transformer,_x_cd,_y_cd);
         
         if((_y_bd[0]>0.0 && _y_cd[0]>0.0)||(_y_bd[0]<0.0 && _y_cd[0]<0.0))//abd agrees with acd
         {
            _refresh=0.5*(_y_bd[0]+_y_cd[0]);
         }
      }
      
      return(_refresh);
   }
//+------------------------------------------------------------------+
